#! /usr/bin/env python

# Copyright (C) 2005  Jean-Claude Rimbault & Optiman
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import Cookie, glob, imaplib, rfc822, os, re, string, time
import traceback, urllib, urlparse, sys, locale
import gc
gc.disable()
from pythonite.context import Context, Option
from pythonite.session import Session
from pythonite.objects.datetime_obj import DateTime
from pythonite.templating import expand
from pythonite.cache import cache_check, cache_save
from pythonite.utils import *

__version__ = '0.6'
__sig__ = '<meta name="generator" content="pythonite v%s  (http://www.pythonite.org/)">' % __version__

def handle_request(req):
    """ handle http client request and returns html page """
    log(1, 'req.uri='+req.uri)
    log(1, 'rootdir='+req.rootdir)
    t0 = time.clock()
    req.version = __version__
    # set up a few objects
    req.date = DateTime(req.request_time)
    req.option = Option(req)
    req.session = Session(req)
    req.errors = []
    if req.session.language:
        req.accept_language = [req.session.language]
    else:
        req.accept_language = []
        try:
            accept_language = req.headers_in['Accept-Language']
        except KeyError:
            accept_language = ''
        for lang in accept_language.split(','):
            req.accept_language.append(lang.split(';')[0])
    log(1, 'accept_language=%s' % req.accept_language)
    data = cache_check(req) # optional cache check
    if not data:
        skin = req.option.skin
        req.query = Empty()
        for field in req.fields:
            if field.name.startswith('py:'):
                cmd = field.name[3:]
                if cmd == 'skin': ### security check to add
                    skin = os.path.join(config.skindir, field.value+config.extension)
                elif cmd == 'language':
                    req.session.language = field.value
                    req.accept_language = [field.value]
            else:
                setattr(req.query, field.name, field.value)
        req.content_type = "text/html"
        req.data = None
        for lang in req.accept_language:
            if '-' in lang:
                lang = lang[:2] + '_' + lang[3:].upper()
            try:
                locale.setlocale(locale.LC_TIME, locale.normalize(lang))
            except locale.Error:
                pass
            else:
                break
            if '_' in lang:
                lang = lang[:2]
                try:
                    locale.setlocale(locale.LC_TIME, locale.normalize(lang))
                except locale.Error:
                    pass
                else:
                    break
        else:
            log(1, 'cannot find locale for %s' % ' '.join(req.accept_language))
        if not skin:
            data = open(req.filename).read()
        else:
            context = Context(req)
            context.set('query', req.query)
            try:
                mainpath = path_find(os.path.dirname(req.filename), skin, req)
                main = open(mainpath).read()
            except IOError:
                main = ''
            try:
                if not main:
                    data = open(req.filename).read()
                else:
                    data = expand(main, context)
            except:
                data = reporterr()
        cache_save(req, data)
        gentime = '%.2f' % (time.clock() - t0)
    else:
        gentime = '*%.2f' % (time.clock() - t0)
    data = data.replace('&gentime;', gentime) # update generation time
    data = data.replace('<head>', '<head>\n'+__sig__) # update meta generator
    for err in req.errors:
        data += '<!-- '+err+' -->\n'
    req.session.save()
    log(1, 'return OK')
    return data
