#! /usr/bin/env python

# Copyright (C) 2005  Jean-Claude Rimbault & Optiman
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

from pythonite.context import register_obj, register_ext, type_registry, makefile
from pythonite.objects.file_obj import File
from pythonite.utils import log
from pythonite import config
import fnmatch, os

class Directory(File):
    objtype = 'directory'
    def __len__(self):
        return len(self.all)
    def __getitem__(self, i):
        return self.all[i]
    def __iter__(self):
        return iter(self.all)
    def list(self, pattern='', path='', type=''):
        if path:
            if path.startswith('/'):
                req = self.context.get('request')
                path = os.path.join(req.rootdir, path[1:])
            directory = Directory(os.path.join(self.path, path), self.context)
        else:
            directory = self
        if type:
            extensions = type_registry[type]
            flag = 0
            if '' in extensions:
                flag += 1
            if '/' in extensions:
                flag += 2
        else:
            extensions = {}
            flag = 3
        res = []
        files = os.listdir(directory.path)
        files.sort()
        for file in files:
            if file.startswith('.') or file.endswith(config.index):
                continue
            if pattern and not fnmatch.fnmatch(file, pattern):
                continue
            if flag:
                path = os.path.join(directory.path, file)
                if os.path.isdir(path):
                    if flag & 2 and not path.endswith('/'+config.skindir):
                       res.append(makefile(path, self.context))
                else:
                    if flag & 1:
                       res.append(makefile(path, self.context))
            else:
                prefix, suffix = os.path.splitext(file)
                suffix = suffix.lower()
                if not extensions or suffix in extensions:
                    res.append(makefile(os.path.join(directory.path, file), self.context))
        return res
    def file(self, name):
        path = os.path.join(self.path, name)
        return makefile(path, self.context)
    def get_url(self):
        path = File.get_url(self)
        return path+'/'
    def get_obj(self, name):
        for file in os.listdir(self.path):
            prefix, suffix = os.path.splitext(file)
            if prefix.lower() == name:
                return makefile(os.path.join(self.path, file), self.context)
        raise AttributeError, name
    def get_link(self):
        return Link(self.url, self.name)

register_ext('/', Directory)
register_obj('directory', Directory, 'os.path.dirname(request.filename)')
register_obj('root', Directory, 'request.rootdir')
