#! /usr/bin/env python

# Copyright (C) 2005  Jean-Claude Rimbault & Optiman
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import glob, os, sys, time, traceback
import config

def log(level, msg):
    if level > config.loglevel:
	return
    try:
	1/0
    except:
	e,x,tb = sys.exc_info()
	func = tb.tb_frame.f_back.f_code.co_name
	lineno = tb.tb_frame.f_back.f_code.co_firstlineno
    try:
        timestamp = '%04d-%02d-%02dT%02d:%02d:%02d' % time.localtime()[:6]
	open('/var/log/pythonite', 'a').write('%s %s:%s: %s\n' % (timestamp, func, lineno, msg))
    except IOError:
	pass

def reporterr():
    exc, info, tb = sys.exc_info()
    data = '<HTML><HEAD><TITLE>Error: %s</TITLE></HEAD><BODY>' % exc
    data += '<PRE>'
    data += ''.join(traceback.format_exception(exc, info, tb))
    data += '</PRE>'
    data += '</BODY></HTML>'
    return data

def relpath(new, old):
    old = old.split('/')
    new = new.split('/')
    for i in range(min(len(new), len(old))):
        if new[i] != old[i]:
            break
    else:
        i += 1
    up = (len(old) - i) * '../'
    down = '/'.join(new[i:])
    return os.path.normpath(up + down)

def expand_language(dir, name, req):
    """ expand filename according to language """
    defpath = os.path.join(dir, name)
    deflang = req.option.language
    basename, ext = os.path.splitext(name)
    for lang in req.accept_language:
        if lang == deflang and os.path.exists(defpath):
            log(1, 'found default language "%s" file %s' % (lang, defpath))
            return defpath
        path = os.path.join(dir, basename+'.'+lang+ext)
        if os.path.exists(path):
            log(1, 'found language "%s" file %s' % (lang, path))
            return path
        if '-' in deflang and lang == deflang.split('-')[0] and os.path.exists(defpath):
            log(1, 'found default language "%s" file %s matching "%s"' % (deflang, defpath, lang))
            return defpath
        if '-' in lang:
            lang = lang.split('-')[0]
            if lang == deflang and os.path.exists(deflang):
                log(1, 'found default language "%s" file %s matching "%s"' % (deflang, defpath, lang))
                return defpath
        path = os.path.join(dir, basename+'.'+lang+'*'+ext)
        files = glob.glob(path)
        if files:
            log(1, 'found file %s matching "%s"' % (files[0], lang))
            return files[0]
    return defpath

def path_find(dir, name, req):
    """ search for a file starting from path and climbing the tree backwards """
    log(1, "dir=%s name=%s" % (dir, name))
    while True:
        path = expand_language(dir, name, req)
        if os.path.exists(path):
            break
        updir = os.path.dirname(dir)
        if updir != dir: # this could be a security hole but practically it's OK
            dir = updir
        else:
            log(1, '%s not found!' % path)
            return path
    log(1, 'found %s' % path)
    return path

def re_find(regexp, data):
    match = regexp.search(data)
    if match:
        return match.group(1)
    else:
        return ''

def sort(items, field=None, reverse=0):
    if field:
        items = [(getattr(item, field), item) for item in items]
    items.sort()
    if field:
        items = [item[1] for item in items]
    if reverse:
        items.reverse()
    return items

def dump(req, name, obj, table=0):
    req.write("<TABLE>\n")
    req.write("<CAPTION><B>%s</B></CAPTION>\n" % name)
    if table:
        keys = obj.keys()
    else:
        keys = dir(obj)
    for key in keys:
        req.write("<TR>\n")
        req.write(" <TD>%s\n" % key)
        #if table:
        #    elem = obj[key]
        #else:
        #    elem = getattr(obj, key)
        elem = ''
        req.write(" <TD>%s\n" % type(elem).__name__)
        req.write(" <TD>%s\n" % str(elem))
    req.write("</TABLE>\n")

class Empty:
    def __getattr__(self, attr):
        if attr.startswith('_'):
            raise AttributeError, attr
        return ''

